import 'package:backend_dart/domain/data/database.dart';
import 'package:backend_dart/domain/entities/project.dart';
import 'package:backend_dart/domain/interface/error.dart';
import 'package:backend_dart/domain/repository/project_repository.dart';
import 'package:fpdart/fpdart.dart';

class ProjectRepositoryImpl implements ProjectRepository {
  final IDatabase database;
  ProjectRepositoryImpl(this.database);

  @override
  TaskEither<IError, Project> create(ProjectCreate project) {
    return database.projects
        .generateId()
        .map((id) => project.copyWith(id: id))
        .flatMap(database.projects.create);
  }

  @override
  TaskEither<IError, Project> findById(String id) {
    return database.projects.findById(id);
  }

  @override
  TaskEither<IError, Project> update(ProjectUpdate project) {
    return database.projects.update(project);
  }

  @override
  TaskEither<IError, void> delete(String id) {
    return database.projects.delete(id);
  }

  @override
  TaskEither<IError, List<Project>> findAll() {
    return database.projects.findAll();
  }
}
