import 'package:backend_dart/domain/data/database.dart';
import 'package:backend_dart/domain/entities/project_task.dart';
import 'package:backend_dart/domain/interface/error.dart';
import 'package:backend_dart/domain/repository/project_task_repository.dart';
import 'package:fpdart/fpdart.dart';

class ProjectTaskRepositoryImpl implements ProjectTaskRepository {
  final IDatabase database;
  ProjectTaskRepositoryImpl(this.database);

  @override
  TaskEither<IError, ProjectTask> create(ProjectTaskCreate task) {
    return database.tasks
        .generateId()
        .map((id) => task.copyWith(id: id))
        .flatMap(database.tasks.create);
  }

  @override
  TaskEither<IError, ProjectTask> findById(String id) {
    return database.tasks.findById(id);
  }

  @override
  TaskEither<IError, List<ProjectTask>> findByProjectId(String projectId) {
    return database.tasks.findByProjectId(projectId);
  }

  @override
  TaskEither<IError, ProjectTask> update(ProjectTaskUpdate task) {
    return database.tasks.update(task);
  }

  @override
  TaskEither<IError, void> delete(String id) {
    return database.tasks.delete(id);
  }

  @override
  TaskEither<IError, List<ProjectTask>> findAll() {
    return database.tasks.findAll();
  }
}
