import 'package:backend_dart/domain/data/database.dart';
import 'package:backend_dart/domain/entities/time_entry.dart';
import 'package:backend_dart/domain/interface/error.dart';
import 'package:backend_dart/domain/repository/time_entry_repository.dart';
import 'package:fpdart/fpdart.dart';

class TimeEntryRepositoryImpl implements TimeEntryRepository {
  final IDatabase database;
  TimeEntryRepositoryImpl(this.database);

  @override
  TaskEither<IError, TimeEntry> create(TimeEntryCreate timeEntry) {
    return database.timeEntries
        .generateId()
        .map((id) => timeEntry.copyWith(id: id))
        .flatMap(database.timeEntries.create);
  }

  @override
  TaskEither<IError, TimeEntry> findById(String id) {
    return database.timeEntries.findById(id);
  }

  @override
  TaskEither<IError, List<TimeEntry>> findByUserId(String userId) {
    return database.timeEntries.findByUserId(userId);
  }

  @override
  TaskEither<IError, List<TimeEntry>> findByProjectId(String projectId) {
    return database.timeEntries.findByProjectId(projectId);
  }

  @override
  TaskEither<IError, TimeEntry> update(TimeEntryUpdate timeEntry) {
    return database.timeEntries.update(timeEntry);
  }

  @override
  TaskEither<IError, void> delete(String id) {
    return database.timeEntries.delete(id);
  }

  @override
  TaskEither<IError, List<TimeEntry>> findAll() {
    return database.timeEntries.findAll();
  }
}
