import 'package:backend_dart/application/service/dto/project_dto.dart';
import 'package:backend_dart/domain/entities/project.dart';
import 'package:backend_dart/domain/interface/error.dart';
import 'package:fpdart/fpdart.dart';

class ProjectDtoMapper {
  TaskEither<IError, ProjectDto> to(Project entity) => TaskEither.of(ProjectDto(
        id: entity.id,
        name: entity.name,
        description: entity.description,
        clientId: entity.clientId,
        userId: entity.userId,
        createdAt: entity.createdAt,
        updatedAt: entity.updatedAt,
      ));

  TaskEither<IError, List<ProjectDto>> listTo(Iterable<Project> origins) {
    return TaskEither.traverseList(origins.toList(), to);
  }

  TaskEither<IError, ProjectCreate> fromCreateTo(ProjectCreateDto origin) =>
      TaskEither.of(ProjectCreate(
        name: origin.name,
        description: origin.description,
        clientId: origin.clientId,
        userId: origin.userId,
      ));

  TaskEither<IError, ProjectUpdate> fromUpdateTo(
          ProjectUpdateDto origin, String id) =>
      TaskEither.of(ProjectUpdate(
        id: id,
        name: origin.name,
        description: origin.description,
        clientId: origin.clientId,
        userId: origin.userId,
      ));
}
