import 'package:backend_dart/domain/entities/project.dart';
import 'package:backend_dart/domain/interface/error.dart';
import 'package:fpdart/fpdart.dart';

/// Interface for managing project data interactions.
abstract class ProjectDataSource {
  /// Creates a new project in the data source.
  ///
  /// Throws an error if the creation fails.
  TaskEither<IError, Project> create(ProjectCreate project);

  /// Retrieves a project by its unique ID.
  ///
  /// Returns `null` if no project is found.
  TaskEither<IError, Project> findById(String id);

  /// Retrieves all projects associated with a specific user.
  ///
  /// Returns an empty list if no projects are found.
  TaskEither<IError, List<Project>> findByUserId(String userId);

  /// Updates an existing project in the data source.
  ///
  /// Throws an error if the update fails.
  TaskEither<IError, Project> update(ProjectUpdate project);

  /// Deletes a project by its unique ID.
  ///
  /// Throws an error if the deletion fails.
  TaskEither<IError, Project> delete(String id);

  /// Retrieves all projects in the data source.
  ///
  /// Returns an empty list if no projects exist.
  TaskEither<IError, List<Project>> findAll();

  /// Generates a new unique ID for a task.
  TaskEither<IError, String> generateId();
}
