import 'package:backend_dart/domain/entities/project_task.dart';
import 'package:backend_dart/domain/interface/error.dart';
import 'package:fpdart/fpdart.dart';

/// Interface for managing task data interactions.
abstract class ProjectTaskDataSource {
  /// Creates a new task in the data source.
  ///
  /// Throws an error if the creation fails.
  TaskEither<IError, ProjectTask> create(ProjectTaskCreate task);

  /// Retrieves a task by its unique ID.
  ///
  /// Returns `null` if no task is found.
  TaskEither<IError, ProjectTask> findById(String id);

  /// Retrieves all tasks for a specific project.
  ///
  /// Returns an empty list if no tasks are found.
  TaskEither<IError, List<ProjectTask>> findByProjectId(String projectId);

  /// Updates an existing task in the data source.
  ///
  /// Throws an error if the update fails.
  TaskEither<IError, ProjectTask> update(ProjectTaskUpdate task);

  /// Deletes a task by its unique ID.
  ///
  /// Throws an error if the deletion fails.
  TaskEither<IError, ProjectTask> delete(String id);

  /// Retrieves all tasks in the data source.
  ///
  /// Returns an empty list if no tasks exist.
  TaskEither<IError, List<ProjectTask>> findAll();

  /// Generates a new unique ID for a task.
  TaskEither<IError, String> generateId();
}
