import 'package:backend_dart/domain/entities/time_entry.dart';
import 'package:backend_dart/domain/interface/error.dart';
import 'package:fpdart/fpdart.dart';

/// Interface for managing time entry data interactions.
abstract class TimeEntryDataSource {
  /// Creates a new time entry in the data source.
  ///
  /// Throws an error if the creation fails.
  TaskEither<IError, TimeEntry> create(TimeEntryCreate timeEntry);

  /// Retrieves a time entry by its unique ID.
  ///
  /// Returns `null` if no time entry is found.
  TaskEither<IError, TimeEntry> findById(String id);

  /// Retrieves all time entries for a specific user.
  ///
  /// Returns an empty list if no time entries are found.
  TaskEither<IError, List<TimeEntry>> findByUserId(String userId);

  /// Retrieves all time entries for a specific project.
  ///
  /// Returns an empty list if no time entries are found.
  TaskEither<IError, List<TimeEntry>> findByProjectId(String projectId);

  /// Updates an existing time entry in the data source.
  ///
  /// Throws an error if the update fails.
  TaskEither<IError, TimeEntry> update(TimeEntryUpdate timeEntry);

  /// Deletes a time entry by its unique ID.
  ///
  /// Throws an error if the deletion fails.
  TaskEither<IError, TimeEntry> delete(String id);

  /// Retrieves all time entries in the data source.
  ///
  /// Returns an empty list if no time entries exist.
  TaskEither<IError, List<TimeEntry>> findAll();

  /// Generates a new unique ID for a time entry.
  TaskEither<IError, String> generateId();
}
