import 'package:backend_dart/domain/interface/error.dart';
import 'package:freezed_annotation/freezed_annotation.dart';
import 'error_code.dart';

part 'app_error.freezed.dart';

@freezed
class AppError with _$AppError implements IError {
  const factory AppError({
    required ErrorCode code,
    required String message,
    Map<String, String>? details,
    StackTrace? stackTrace,
    Object? originalError,
  }) = _AppError;

  factory AppError.fromException(Exception e) {
    return AppError(
      code: ErrorCode.exception,
      message: e.toString(),
      originalError: e,
    );
  }

  factory AppError.fromErrorCode(ErrorCode code, {String? message}) {
    return AppError(
        code: code,
        message: message ?? "No message",
        stackTrace: StackTrace.current);
  }

  factory AppError.notFound(String? message) {
    return AppError.fromErrorCode(ErrorCode.notFound, message: message);
  }

  factory AppError.mappingError({String? message}) {
    return AppError.fromErrorCode(ErrorCode.mappingError, message: message);
  }

  factory AppError.databaseError({String? message}) {
    return AppError.fromErrorCode(ErrorCode.databaseError, message: message);
  }

  factory AppError.validationError({String? message}) {
    return AppError.fromErrorCode(ErrorCode.validationError, message: message);
  }

  factory AppError.inputError({String? message}) {
    return AppError.fromErrorCode(ErrorCode.inputError, message: message);
  }
}
