import 'package:fpdart/fpdart.dart';
import 'package:backend_dart/domain/entities/project_task.dart';
import 'package:backend_dart/domain/interface/error.dart';

abstract class ProjectTaskRepository {
  /// Creates a new project task.
  TaskEither<IError, ProjectTask> create(ProjectTaskCreate task);

  /// Finds a project task by its unique ID.
  TaskEither<IError, ProjectTask> findById(String id);

  /// Finds all tasks for a specific project.
  TaskEither<IError, List<ProjectTask>> findByProjectId(String projectId);

  /// Updates an existing project task.
  TaskEither<IError, ProjectTask> update(ProjectTaskUpdate task);

  /// Deletes a project task by its unique ID.
  TaskEither<IError, void> delete(String id);

  /// Finds all project tasks.
  TaskEither<IError, List<ProjectTask>> findAll();
}
