import 'package:fpdart/fpdart.dart';
import 'package:backend_dart/domain/entities/time_entry.dart';
import 'package:backend_dart/domain/interface/error.dart';

abstract class TimeEntryRepository {
  /// Creates a new time entry.
  TaskEither<IError, TimeEntry> create(TimeEntryCreate timeEntry);

  /// Finds a time entry by its unique ID.
  TaskEither<IError, TimeEntry> findById(String id);

  /// Finds all time entries for a specific user.
  TaskEither<IError, List<TimeEntry>> findByUserId(String userId);

  /// Finds all time entries for a specific project.
  TaskEither<IError, List<TimeEntry>> findByProjectId(String projectId);

  /// Updates an existing time entry.
  TaskEither<IError, TimeEntry> update(TimeEntryUpdate timeEntry);

  /// Deletes a time entry by its unique ID.
  TaskEither<IError, void> delete(String id);

  /// Finds all time entries.
  TaskEither<IError, List<TimeEntry>> findAll();
}
