import 'package:backend_dart/common/extensions.dart';
import 'package:backend_dart/domain/entities/project.dart';
import 'package:backend_dart/domain/interface/error.dart';
import 'package:backend_dart/infrastructure/persistence/db/model.dart';
import 'package:backend_dart/infrastructure/persistence/db/prisma.dart';
import 'package:fpdart/fpdart.dart';
import 'package:orm/orm.dart';

class ProjectDboMapper {
  TaskEither<IError, Project> fromDbo(ProjectDbo dbo) {
    return TaskEither.of(Project(
      id: dbo.id!,
      name: dbo.name!,
      description: dbo.description,
      clientId: dbo.clientId,
      userId: dbo.userId!,
      createdAt: dbo.createdAt!,
      updatedAt: dbo.updatedAt!,
    ));
  }

  TaskEither<IError, List<Project>> listFrom(Iterable<ProjectDbo> dbos) {
    return TaskEither.traverseList(dbos.toList(), fromDbo);
  }

  TaskEither<IError, ProjectDboCreateInput> fromCreatetoDbo(
      ProjectCreate project) {
    return TaskEither.of(ProjectDboCreateInput(
      id: project.id,
      name: project.name,
      description: project.description.let(PrismaUnion.$1),
      clientId: project.clientId.let(PrismaUnion.$1),
      user: UserDboCreateNestedOneWithoutProjectsInput(
        connect: UserDboWhereUniqueInput(id: project.userId),
      ),
    ));
  }

  TaskEither<IError, ProjectDboUpdateInput> fromUpdateToDbo(
      ProjectUpdate project) {
    return TaskEither.of(ProjectDboUpdateInput(
      id: PrismaUnion.$1(project.id),
      name: project.name.let(PrismaUnion.$1),
      description: project.description.let(PrismaUnion.$1),
      clientId: project.clientId.let(PrismaUnion.$1),
      user: project.userId.let(
        (userId) => UserDboUpdateOneRequiredWithoutProjectsNestedInput(
          connect: UserDboWhereUniqueInput(id: userId),
        ),
      ),
    ));
  }
}
