import 'package:backend_dart/common/extensions.dart';
import 'package:backend_dart/domain/entities/project_task.dart';
import 'package:backend_dart/domain/interface/error.dart';
import 'package:backend_dart/infrastructure/persistence/db/model.dart';
import 'package:backend_dart/infrastructure/persistence/db/prisma.dart';
import 'package:fpdart/fpdart.dart';
import 'package:orm/orm.dart';

class ProjectTaskDboMapper {
  TaskEither<IError, ProjectTask> from(ProjectTaskDbo target) =>
      TaskEither.of(ProjectTask(
        id: target.id!,
        name: target.name!,
        description: target.description,
        projectId: target.projectId!,
        createdAt: target.createdAt!,
        updatedAt: target.updatedAt!,
      ));

  TaskEither<IError, List<ProjectTask>> listFrom(
      Iterable<ProjectTaskDbo> targets) {
    return TaskEither.traverseList(targets.toList(), from);
  }

  TaskEither<IError, ProjectTaskDboCreateInput> fromCreateTo(
          ProjectTaskCreate origin) =>
      TaskEither.of(ProjectTaskDboCreateInput(
        id: origin.id,
        name: origin.name,
        description: origin.description.let(PrismaUnion.$1),
        project: ProjectDboCreateNestedOneWithoutTasksInput(
          connect: ProjectDboWhereUniqueInput(
            id: origin.projectId,
          ),
        ),
      ));

  TaskEither<IError, ProjectTaskDboUpdateInput> fromUpdateTo(
          ProjectTaskUpdate origin) =>
      TaskEither.of(ProjectTaskDboUpdateInput(
        id: PrismaUnion.$1(origin.id),
        name: origin.name?.let(PrismaUnion.$1),
        description: origin.description?.let(PrismaUnion.$1),
        project: origin.projectId?.let(
          (projectId) => ProjectDboUpdateOneRequiredWithoutTasksNestedInput(
            connect: ProjectDboWhereUniqueInput(
              id: projectId,
            ),
          ),
        ),
      ));
}
