import 'package:backend_dart/common/extensions.dart';
import 'package:backend_dart/domain/entities/time_entry.dart';
import 'package:backend_dart/domain/interface/error.dart';
import 'package:backend_dart/infrastructure/persistence/db/model.dart';
import 'package:backend_dart/infrastructure/persistence/db/prisma.dart';
import 'package:fpdart/fpdart.dart';
import 'package:orm/orm.dart';

class TimeEntryDboMapper {
  TaskEither<IError, TimeEntry> from(TimeEntryDbo target) => TaskEither.of(
        TimeEntry(
          id: target.id!,
          startTime: target.startTime!,
          endTime: target.endTime!,
          description: target.description,
          userId: target.userId!,
          projectId: target.projectId!,
          createdAt: target.createdAt!,
          updatedAt: target.updatedAt!,
        ),
      );

  TaskEither<IError, List<TimeEntry>> listFrom(Iterable<TimeEntryDbo> targets) {
    return TaskEither.traverseList(targets.toList(), from);
  }

  TaskEither<IError, TimeEntryDboCreateInput> fromCreateTo(
          TimeEntryCreate origin) =>
      TaskEither.of(TimeEntryDboCreateInput(
        id: origin.id,
        startTime: origin.startTime,
        endTime: origin.endTime.let(PrismaUnion.$1),
        description: origin.description.let(PrismaUnion.$1),
        user: UserDboCreateNestedOneWithoutTimeEntriesInput(
          connect: UserDboWhereUniqueInput(
            id: origin.userId,
          ),
        ),
        project: ProjectDboCreateNestedOneWithoutTimeEntriesInput(
          connect: ProjectDboWhereUniqueInput(
            id: origin.projectId,
          ),
        ),
      ));

  TaskEither<IError, TimeEntryDboUpdateInput> fromUpdateTo(
          TimeEntryUpdate origin) =>
      TaskEither.of(TimeEntryDboUpdateInput(
        id: PrismaUnion.$1(origin.id),
        startTime: origin.startTime?.let(PrismaUnion.$1),
        endTime: origin.endTime?.let(PrismaUnion.$1),
        description: origin.description?.let(PrismaUnion.$1),
        user: origin.userId?.let(
          (userId) => UserDboUpdateOneRequiredWithoutTimeEntriesNestedInput(
            connect: UserDboWhereUniqueInput(
              id: userId,
            ),
          ),
        ),
        project: origin.projectId?.let(
          (projectId) =>
              ProjectDboUpdateOneRequiredWithoutTimeEntriesNestedInput(
            connect: ProjectDboWhereUniqueInput(
              id: projectId,
            ),
          ),
        ),
      ));
}
