import 'package:backend_dart/common/extensions.dart';
import 'package:backend_dart/common/secure_hash.dart';
import 'package:backend_dart/domain/entities/user.dart';
import 'package:backend_dart/domain/interface/error.dart';
import 'package:backend_dart/infrastructure/persistence/db/model.dart';
import 'package:backend_dart/infrastructure/persistence/db/prisma.dart';
import 'package:fpdart/fpdart.dart';
import 'package:orm/orm.dart';

class UserDboMapper {
  TaskEither<IError, User> from(UserDbo target) => TaskEither.of(User(
        id: target.id!,
        name: target.name!,
        email: target.email!,
        passwordHash: target.password,
        createdAt: target.createdAt,
        updatedAt: target.updatedAt,
      ));

  TaskEither<IError, List<User>> listFrom(Iterable<UserDbo> targets) {
    return TaskEither.traverseList(targets.toList(), from);
  }

  TaskEither<IError, UserDboUpdateInput> fromUpdateTo(UserUpdate origin) =>
      TaskEither.of(UserDboUpdateInput(
        id: PrismaUnion.$1(origin.id),
        name: origin.name.let(PrismaUnion.$1),
        email: origin.email.let(PrismaUnion.$1),
        password: origin.password.let(generateSecureHash).let(PrismaUnion.$1),
      ));

  TaskEither<IError, UserDboCreateInput> fromCreateTo(UserCreate origin) =>
      TaskEither.of(UserDboCreateInput(
        id: origin.id,
        name: origin.name,
        email: origin.email,
        password: generateSecureHash(origin.password),
      ));
}
