import 'package:backend_dart/domain/data/database.dart';
import 'package:backend_dart/domain/data/project_data_source.dart';
import 'package:backend_dart/domain/data/project_task_data_source.dart';
import 'package:backend_dart/domain/data/time_entry_data_source.dart';
import 'package:backend_dart/infrastructure/persistence/db/client.dart';
import 'package:backend_dart/infrastructure/persistence/prisma_project_data_source.dart';
import 'package:backend_dart/infrastructure/persistence/prisma_project_task_data_source.dart';
import 'package:backend_dart/infrastructure/persistence/prisma_time_entry_data_source.dart';
import 'package:backend_dart/infrastructure/persistence/prisma_user_data_source.dart';

class PrismaDatabase implements IDatabase {
  final prisma = PrismaClient();
  late final PrismaUserDataSource _users;
  late final TimeEntryDataSource _timeEntries;
  late final ProjectTaskDataSource _tasks;
  late final ProjectDataSource _projects;

  PrismaDatabase() {
    _users = PrismaUserDataSource(prisma);
    _timeEntries = PrismaTimeEntryDataSource(prisma);
    _tasks = PrismaProjectTaskDataSource(prisma);
    _projects = PrismaProjectDataSource(prisma);
    print('Database initialized');
  }

  @override
  get users => _users;
  @override
  get timeEntries => _timeEntries;
  @override
  get tasks => _tasks;
  @override
  get projects => _projects;
  @override
  Future<void> close() {
    return Future.value();
  }
}
